Enum CtorOf
pub enum CtorOf {
Struct,
Variant,
}Expand description
Encodes if a DefKind::Ctor is the constructor of an enum variant or a struct.
Variants
Struct
This DefKind::Ctor is a synthesized constructor of a tuple or unit struct.
Variant
This DefKind::Ctor is a synthesized constructor of a tuple or unit variant.