pub enum DefIdBase {
Real(DefId),
Promoted(DefId, PromotedId),
ImplAssocItem(VirtualImplAssocItem),
Synthetic(SyntheticItem),
}Variants
Real(DefId)
Promoted(DefId, PromotedId)
ImplAssocItem(VirtualImplAssocItem)
This represents the context made of the trait impl generics plus the associated item generics declared in the trait. We use that context to trait solve the mapping from declared method generics to implemented method generics.
Synthetic(SyntheticItem)
A completely fictitious item, we use this for arrays, slices and tuples to make monomorphization and other shenanigans easier.