ImplAssocItemValue

Enum ImplAssocItemValue 

pub enum ImplAssocItemValue {
    Provided {
        def_id: DefId,
        is_override: bool,
    },
    DefaultedTy {
        ty: Option<Ty>,
    },
    DefaultedFn {
        sig: Option<PolyFnSig>,
    },
    DefaultedConst,
}

Variants



Provided

The item is provided by the trait impl.

Fields

def_id: DefId

The definition of the item in the trait impl. This is an AssocTy, AssocFn or AssocConst.

is_override: bool

Whether the trait had a default value for this item (which is therefore overriden).



DefaultedTy

This is an associated type that reuses the trait declaration default.

Fields

ty: Option<Ty>

The default type, with generics properly instantiated. None if the type has generics of its own, because then we’d need to resolve traits but the type doesn’t have its own DefId.



DefaultedFn

This is a non-overriden default method. FIXME: provide properly instantiated generics.

Fields

sig: Option<PolyFnSig>

The signature of the method, if we could translate it. None if the method has generics of its own, because then we’d need to resolve traits but the method doesn’t have its own DefId.



DefaultedConst

This is an associated const that reuses the trait declaration default. The default const value can be found in decl_def.