pub enum ImplExprAtom {
Concrete(ItemRef),
LocalBound {
id: GenericPredicateId,
trait: Binder<TraitRef>,
path: Vec<ImplExprPathChunk>,
},
SelfImpl {
trait: Binder<TraitRef>,
path: Vec<ImplExprPathChunk>,
},
Dyn,
Builtin {
trait_data: BuiltinTraitData,
impl_exprs: Vec<ImplExpr>,
types: Vec<(DefId, Ty, Vec<ImplExpr>)>,
},
Error(String),
}Expand description
The source of a particular trait implementation. Most often this is either Concrete for a
concrete impl Trait for Type {} item, or LocalBound for a context-bound where T: Trait.
Variants
Concrete(ItemRef)
A concrete impl Trait for Type {} item.
LocalBound
A context-bound clause like where T: Trait.
SelfImpl
The implicit Self: Trait clause present inside a trait Trait {} item.
Dyn
dyn Trait is a wrapped value with a virtual table for trait
Trait. In other words, a value dyn Trait is a dependent
triple that gathers a type τ, a value of type τ and an
instance of type Trait.
dyn Trait implements Trait using a built-in implementation; this refers to that
built-in implementation.
Builtin
A built-in trait whose implementation is computed by the compiler, such as FnMut. This
morally points to an invisible impl block; as such it contains the information we may
need from one.
Fields
trait_data: BuiltinTraitDataExtra data for the given trait.
Error(String)
An error happened while resolving traits.