SyntheticItem

Enum SyntheticItem 

pub enum SyntheticItem {
    Array,
    Slice,
    Tuple(usize),
}
We create some extra DefIds to represent things that rustc doesn’t have a DefId for. This makes the pipeline much easier to have “real” def_ids for them. We generate fake struct-like items for each of: arrays, slices, and tuples. This makes it easier to emit trait impls for these types, especially with monomorphization. This enum tracks identifies these builtin types.

Variants



Array

Fake ADT representing the [T; N] type.



Slice

Fake ADT representing the [T] type.



Tuple(usize)

Fake ADT representing the length-n tuple (A, B, ...).

Implementations

Source

impl SyntheticItem

pub fn name(&self) -> String

pub fn can_have_generics<'tcx>(&self, s: &impl BaseState<'tcx>) -> bool

pub fn generics_of<'tcx>(&self, s: &impl BaseState<'tcx>) -> &'tcx Generics

pub fn identity_args<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> GenericArgsRef<'tcx>

pub fn param_env<'tcx>(&self, s: &impl BaseState<'tcx>) -> ParamEnv<'tcx>

pub fn predicates_defined_on<'tcx>( &self, s: &impl BaseState<'tcx>, def_id: DefId, direction: PredicateDirection, ) -> ItemPredicates<'tcx, DefId>

pub fn type_of<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> EarlyBinder<'tcx, Ty<'tcx>>

fn data<'tcx>(&self, s: &impl BaseState<'tcx>) -> SyntheticItemData<'tcx>