pub enum TraitProofKind {
Concrete(ItemRef),
LocalBound(GenericPredicateId),
SelfProof,
Dyn,
Builtin {
trait_data: BuiltinTraitData,
proofs: Vec<TraitProof>,
types: Vec<(DefId, Ty, Vec<TraitProof>)>,
},
Derived {
base: TraitProof,
path: TraitProofImpliedPredicate,
},
Error(String),
}Expand description
The source of a particular trait implementation. Most often this is either Concrete for a
concrete impl Trait for Type {} item, or LocalBound for a context-bound where T: Trait.
Variants
Concrete(ItemRef)
A concrete impl Trait for Type {} item.
LocalBound(GenericPredicateId)
A context-bound clause like where T: Trait.
SelfProof
The implicit Self: Trait clause present inside a trait Trait {} item.
Dyn
dyn Trait is a wrapped value with a virtual table for trait
Trait. In other words, a value dyn Trait is a dependent
triple that gathers a type τ, a value of type τ and an
instance of type Trait.
dyn Trait implements Trait using a built-in implementation; this refers to that
built-in implementation.
Builtin
A built-in trait whose implementation is computed by the compiler, such as FnMut. This
morally points to an invisible impl block; as such it contains the information we may
need from one.
Fields
trait_data: BuiltinTraitDataExtra data for the given trait.
proofs: Vec<TraitProof>The trait proofs required to satisfy the implied predicates on the trait declaration.
E.g. since FnMut: FnOnce, a built-in T: FnMut impl would have a proof for
T: FnOnce.
Derived
A predicate implied by base by following path.
Error(String)
An error happened while resolving traits.