pub fn closure_sig<'tcx>(
tcx: TyCtxt<'tcx>,
closure: ClosureArgs<TyCtxt<'tcx>>,
) -> PolyFnSig<'tcx>Expand description
Retrieve a closure’s signature, recovering the regions that rustc erased as fresh bound regions.
pub fn closure_sig<'tcx>(
tcx: TyCtxt<'tcx>,
closure: ClosureArgs<TyCtxt<'tcx>>,
) -> PolyFnSig<'tcx>Retrieve a closure’s signature, recovering the regions that rustc erased as fresh bound regions.