pub struct DefId {
pub(crate) contents: HashConsed<DefIdContents>,
}Expand description
Reflects rustc_hir::def_id::DefId, augmented to also give ids to promoted constants (which
have their own ad-hoc numbering scheme in rustc for now).
Fields
contents: HashConsed<DefIdContents>Implementations
Sourceimpl DefId
impl DefId
Sourcepub fn as_real_def_id(&self) -> Option<RDefId>
pub fn as_real_def_id(&self) -> Option<RDefId>
The rustc def_id corresponding to this item, if there is one. Promoted constants don’t have a rustc def_id.
Sourcepub fn real_rust_def_id(&self) -> RDefId
pub fn real_rust_def_id(&self) -> RDefId
The rustc def_id of this item. Panics if this is not a real rustc item.
Sourcepub fn as_real_or_promoted(&self) -> Option<RDefId>
pub fn as_real_or_promoted(&self) -> Option<RDefId>
The def_id of this item or its parent if this is a promoted constant.
pub fn promoted_id(&self) -> Option<PromotedId>
Sourcepub fn as_synthetic<'tcx>(
&self,
_s: &impl BaseState<'tcx>,
) -> Option<SyntheticItem>
pub fn as_synthetic<'tcx>( &self, _s: &impl BaseState<'tcx>, ) -> Option<SyntheticItem>
Returns the SyntheticItem encoded by this hax DefId, if any.
pub fn is_local(&self) -> bool
pub fn is_typeck_child<'tcx>(&self, s: &impl BaseState<'tcx>) -> bool
fn make<'tcx, S: BaseState<'tcx>>(s: &S, def_id: RDefId) -> Self
pub fn make_synthetic<'tcx, S: BaseState<'tcx>>( s: &S, synthetic: SyntheticItem, ) -> Self
Sourcepub fn make_promoted_child<'tcx, S: BaseState<'tcx>>(
&self,
s: &S,
promoted_id: PromotedId,
) -> Self
pub fn make_promoted_child<'tcx, S: BaseState<'tcx>>( &self, s: &S, promoted_id: PromotedId, ) -> Self
Construct a hax DefId for the nth promoted constant of the current item. That DefId has
no corresponding rustc DefId.
pub fn make_assoc_item_impl<'tcx, S: BaseState<'tcx>>( s: &S, vitem: VirtualImplAssocItem, ) -> Self
Sourceimpl DefId
impl DefId
fn crate_name_and_disambig<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> (Symbol, u32)
pub fn crate_name<'tcx>(&self, s: &impl BaseState<'tcx>) -> Symbol
Sourcepub fn def_span<'tcx>(&self, s: &impl BaseState<'tcx>) -> Span
pub fn def_span<'tcx>(&self, s: &impl BaseState<'tcx>) -> Span
Get the span of the definition of this item. This is the span used in diagnostics when referring to the item.
Sourcepub fn path_item<'tcx>(
&self,
s: &impl BaseState<'tcx>,
) -> DisambiguatedDefPathItem
pub fn path_item<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> DisambiguatedDefPathItem
The PathItem corresponding to this item.
pub fn parent<'tcx>(&self, s: &impl BaseState<'tcx>) -> Option<DefId>
Sourceimpl DefId
impl DefId
pub fn can_have_generics<'tcx>(&self, s: &impl BaseState<'tcx>) -> bool
pub fn generics_of<'tcx>(&self, s: &impl BaseState<'tcx>) -> &'tcx Generics
pub fn identity_args<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> GenericArgsRef<'tcx>
pub fn param_env<'tcx>(&self, s: &impl BaseState<'tcx>) -> ParamEnv<'tcx>
pub fn typing_env<'tcx>(&self, s: &impl BaseState<'tcx>) -> TypingEnv<'tcx>
pub fn required_predicates<'tcx, S: BaseState<'tcx>>( &self, s: &S, ) -> ItemPredicates<'tcx, Self>
pub fn type_of<'tcx, S: BaseState<'tcx>>( &self, s: &S, ) -> EarlyBinder<'tcx, Ty<'tcx>>