FullDef

Struct FullDef 

pub struct FullDef {
    pub this: ItemRef,
    pub span: Span,
    pub source_span: Option<Span>,
    pub source_text: Option<String>,
    pub attributes: Vec<Attribute>,
    pub visibility: Option<bool>,
    pub lang_item: Option<Symbol>,
    pub diagnostic_item: Option<Symbol>,
    pub kind: FullDefKind,
}
Gathers a lot of definition information about a rustc_hir::def_id::DefId.

Fields

this: ItemRef

A reference to the current item. If the item was provided with generic args, they are stored here; otherwise the args are the identity_args for this item.

span: Span

The span of the definition of this item (e.g. for a function this is is signature).

source_span: Option<Span>

The span of the whole definition (including e.g. the function body).

source_text: Option<String>

The text of the whole definition.

attributes: Vec<Attribute>

Attributes on this definition, if applicable.

visibility: Option<bool>

Visibility of the definition, for definitions where this makes sense.

lang_item: Option<Symbol>

If this definition is a lang item, we store the identifier, e.g. sized.

diagnostic_item: Option<Symbol>

If this definition is a diagnostic item, we store the identifier, e.g. box_new.

kind: FullDefKind

Implementations

Source

impl FullDef

pub fn def_id(&self) -> &DefId

pub fn this(&self) -> &ItemRef

Reference to the item itself.

pub fn kind(&self) -> &FullDefKind

pub fn param_env(&self) -> Option<&ParamEnv>

Returns the generics and predicates for definitions that have those.

pub fn typing_parent<'tcx>(&self, s: &impl BaseState<'tcx>) -> Option<ItemRef>

Return the parent of this item if the item inherits the typing context from its parent.

pub fn has_any_generics(&self) -> bool

Whether the item has any generics at all (including parent generics).

pub fn has_own_generics(&self) -> bool

Whether the item has any generics of its own (ignoring parent generics).

pub fn has_own_generics_or_predicates(&self) -> bool

Whether the item has any generics or predicates of its own (ignoring parent generics/predicates).

pub fn nameable_children<'tcx>( &self, s: &impl BaseState<'tcx>, ) -> Vec<(Symbol, DefId)>

Lists the children of this item that can be named, in the way of normal rust paths. For types, this includes inherent items.