pub struct ImplAssocItem {
pub name: Option<Symbol>,
pub decl_def_id: DefId,
pub required_impl_exprs: Vec<ImplExpr>,
pub value: ImplAssocItemValue,
}Expand description
An associated item in a trait impl. This can be an item provided by the trait impl, or an item that reuses the trait decl default value.
Fields
name: Option<Symbol>This is None for RPTITs.
decl_def_id: DefIdThe definition of the item from the trait declaration. This is an AssocTy, AssocFn or
AssocConst.
required_impl_exprs: Vec<ImplExpr>The ImplExprs required to satisfy the predicates on the associated type. E.g.:
ⓘ
trait Foo {
type Type<T>: Clone,
}
impl Foo for () {
type Type<T>: Arc<T>; // would supply an `ImplExpr` for `Arc<T>: Clone`.
}Empty if this item is an associated const or fn.
value: ImplAssocItemValueThe value of the implemented item.
Implementations