PredicateSearcher

Struct PredicateSearcher 

pub struct PredicateSearcher<'tcx> {
    tcx: TyCtxt<'tcx>,
    typing_env: TypingEnv<'tcx>,
    candidates: HashMap<Binder<TyCtxt<'tcx>, TraitRef<TyCtxt<'tcx>>>, Candidate<'tcx>>,
    options: BoundsOptions,
    implicit_self_clause: bool,
}
Stores a set of predicates along with where they came from.

Fields

tcx: TyCtxt<'tcx>typing_env: TypingEnv<'tcx>candidates: HashMap<Binder<TyCtxt<'tcx>, TraitRef<TyCtxt<'tcx>>>, Candidate<'tcx>>options: BoundsOptionsimplicit_self_clause: bool

Implementations



impl<'tcx> PredicateSearcher<'tcx>

pub fn new_for_owner( tcx: TyCtxt<'tcx>, owner_id: DefId, options: &BoundsOptions, ) -> PredicateSearcher<'tcx>

Initialize the elaborator with the predicates accessible within this item.

pub fn insert_bound_predicates( &mut self, preds: impl IntoIterator<Item = ItemPredicate<'tcx>>, )

Insert the bound clauses in the search context. Prefer inserting them all at once as this will give priority to shorter resolution paths. Bound clauses are numbered from 0 in insertion order.

pub fn set_param_env(&mut self, param_env: ParamEnv<'tcx>)

Override the param env; we use this when resolving dyn predicates to add more clauses to the scope.

pub fn resolve( &mut self, tref: &Binder<TyCtxt<'tcx>, TraitRef<TyCtxt<'tcx>>>, warn: &impl Fn(&str), ) -> Result<ImplExpr<'tcx>, String>

Resolve the given trait reference in the local context.

pub fn resolve_item_required_predicates( &mut self, def_id: DefId, generics: &'tcx RawList<(), GenericArg<'tcx>>, warn: &impl Fn(&str), ) -> Result<Vec<ImplExpr<'tcx>>, String>

Resolve the predicates required by the given item.

pub fn resolve_item_implied_predicates( &mut self, def_id: DefId, generics: &'tcx RawList<(), GenericArg<'tcx>>, warn: &impl Fn(&str), ) -> Result<Vec<ImplExpr<'tcx>>, String>

Resolve the predicates implied by the given item.

pub fn resolve_predicates( &mut self, generics: &'tcx RawList<(), GenericArg<'tcx>>, predicates: ItemPredicates<'tcx>, warn: &impl Fn(&str), ) -> Result<Vec<ImplExpr<'tcx>>, String>

Apply the given generics to the provided clauses and resolve the trait references in the current context.