pub struct VirtualTraitImpl {
pub trait_pred: TraitPredicate,
pub implied_impl_exprs: Vec<ImplExpr>,
pub types: Vec<(Ty, Vec<ImplExpr>)>,
}Expand description
Partial data for a trait impl, used for fake trait impls that we generate ourselves such as
FnOnce and Drop impls.
Fields
trait_pred: TraitPredicateThe trait that is implemented by this impl block.
implied_impl_exprs: Vec<ImplExpr>The ImplExprs required to satisfy the predicates on the trait declaration.
types: Vec<(Ty, Vec<ImplExpr>)>The associated types and their predicates, in definition order.