TraitProof

Type Alias TraitProof 

pub type TraitProof = HashConsed<TraitProofContents>;
A TraitProof describes the full data of a trait implementation. Because of generics, this may need to combine several concrete trait implementation items. For example, ((1u8, 2u8), "hello").clone() combines the generic implementation of Clone for (A, B) with the concrete implementations for u8 and &str, represented as a tree.

Aliased Type

pub struct TraitProof(/* private fields */);