ConstantExprKind

Enum ConstantExprKind 

pub enum ConstantExprKind {
Show 13 variants Literal(ConstantLiteral), Adt { kind: VariantKind, fields: Vec<ConstantFieldExpr>, }, Array { fields: Vec<ConstantExpr>, }, Tuple { fields: Vec<ConstantExpr>, }, NamedGlobal(ItemRef), Borrow(ConstantExpr), RawBorrow { mutability: Mutability, arg: ConstantExpr, }, Cast { source: ConstantExpr, }, ConstRef { id: ParamConst, }, FnDef(ItemRef), FnPtr(ItemRef), Memory(Vec<u8>), Todo(String),
}
The subset of [Expr] that corresponds to constants.

Variants



Literal(ConstantLiteral)



Adt



Array

Fields



Tuple

Fields



NamedGlobal(ItemRef)

A top-level or associated constant.

Remark: constants can have generic parameters. Example:

struct V<const N: usize, T> {
  x: [T; N],
}

impl<const N: usize, T> V<N, T> {
  const LEN: usize = N; // This has generics <N, T>
}

impl Foo for Bar {
  const C : usize = 32; // <-
}

If options.inline_anon_consts is false, this is also used for inline const blocks and advanced const generics expressions.



Borrow(ConstantExpr)

A shared reference to a static variable.



RawBorrow

A raw borrow (*const or *mut).

Fields

mutability: Mutability


Cast

A cast <source> as <type>, <type> is stored as the type of the current constant expression. Currently, this is only used to represent lit as *mut T or lit as *const T, where lit is a usize literal.

Fields



ConstRef

Fields



FnDef(ItemRef)

A function definition, corresponding to a particular item. This is a ZST, unlike FnPtr.



FnPtr(ItemRef)

A function pointer. This is an actual pointer to that function.



Memory(Vec<u8>)

A blob of memory containing the byte representation of the value. This can occur when evaluating MIR constants. Interpreting this back to a structured value is left as an exercice to the consumer.



Todo(String)

Implementations

Source

impl ConstantExprKind

pub fn decorate(self, ty: Ty, _span: Span) -> Decorated<Self>