pub enum ImplAssocItemValue {
Provided {
def_id: DefId,
is_override: bool,
},
DefaultedTy {
ty: Option<Ty>,
},
DefaultedFn {
sig: Option<PolyFnSig>,
},
DefaultedConst,
}Variants
Provided
The item is provided by the trait impl.
Fields
DefaultedTy
This is an associated type that reuses the trait declaration default.
Fields
DefaultedFn
This is a non-overriden default method. FIXME: provide properly instantiated generics.
Fields
DefaultedConst
This is an associated const that reuses the trait declaration default. The default const
value can be found in decl_def.