Enum Safety
pub enum Safety {
Unsafe,
Safe,
}Variants
Unsafe
This is the default variant, because the compiler messing up
metadata encoding and failing to encode a Safe flag, means
downstream crates think a thing is Unsafe instead of silently
treating an unsafe thing as safe.
Safe