Struct ItemPredicates
pub struct ItemPredicates<'tcx, Id = DefId> {
direction: PredicateDirection,
next_id: usize,
pub predicates: Vec<ItemPredicate<'tcx, Id>>,
}Fields
direction: PredicateDirectionnext_id: usizepredicates: Vec<ItemPredicate<'tcx, Id>>Implementations
impl<'tcx, Id> ItemPredicates<'tcx, Id>where
Id: ItemId,
impl<'tcx, Id> ItemPredicates<'tcx, Id>where
Id: ItemId,
pub fn new( def_id: Id, direction: PredicateDirection, predicates: impl IntoIterator<Item = (Clause<'tcx>, Span)>, ) -> ItemPredicates<'tcx, Id>
pub fn new_unmapped( span: Span, predicates: impl IntoIterator<Item = Clause<'tcx>>, ) -> ItemPredicates<'tcx, Id>
pub fn defined_on(
tcx: TyCtxt<'tcx>,
state: &<Id as ItemId>::State<'tcx>,
def_id: DefId,
direction: PredicateDirection,
) -> ItemPredicates<'tcx, Id>
pub fn defined_on( tcx: TyCtxt<'tcx>, state: &<Id as ItemId>::State<'tcx>, def_id: DefId, direction: PredicateDirection, ) -> ItemPredicates<'tcx, Id>
Returns a list of type predicates for the definition with id def_id, including inferred
lifetime constraints. This is the basic list of predicates we use for essentially all
items.
pub fn required(
elab_ctx: ElaborationCtx<'tcx, Id>,
state: &<Id as ItemId>::State<'tcx>,
def_id: Id,
) -> ItemPredicates<'tcx, Id>
pub fn required( elab_ctx: ElaborationCtx<'tcx, Id>, state: &<Id as ItemId>::State<'tcx>, def_id: Id, ) -> ItemPredicates<'tcx, Id>
The predicates that must hold to mention this item (ignoring its parents). E.g.
ⓘ
// `U: OtherTrait` is required, `Self: Sized` is implied.
trait Trait<U: OtherTrait>: Sized {
// `T: Clone` is required, `Self::Type<T>: Debug` is implied.
type Type<T: Clone>: Debug;
}If add_drop is true, we add a T: Drop bound for every type generic.
pub fn required_recursively(
elab_ctx: ElaborationCtx<'tcx, Id>,
state: &<Id as ItemId>::State<'tcx>,
def_id: Id,
) -> ItemPredicates<'tcx, Id>
pub fn required_recursively( elab_ctx: ElaborationCtx<'tcx, Id>, state: &<Id as ItemId>::State<'tcx>, def_id: Id, ) -> ItemPredicates<'tcx, Id>
The predicates that must hold to mention this item, including its parents. E.g.
ⓘ
// `U: OtherTrait` is required.
trait Trait<U: OtherTrait> {
// `U: OtherTrait` and `T: Clone` are required.
type Type<T: Clone>;
}pub fn implied(
elab_ctx: ElaborationCtx<'tcx, Id>,
state: &<Id as ItemId>::State<'tcx>,
def_id: Id,
) -> ItemPredicates<'tcx, Id>
pub fn implied( elab_ctx: ElaborationCtx<'tcx, Id>, state: &<Id as ItemId>::State<'tcx>, def_id: Id, ) -> ItemPredicates<'tcx, Id>
The predicates that can be deduced from the presence of this item in a signature. We only
consider predicates implied by traits here, not implied bounds such as &'a T implying T: 'a. E.g.
ⓘ
// `U: OtherTrait` is required, `Self: Sized` is implied.
trait Trait<U: OtherTrait>: Sized {
// `T: Clone` is required, `Self::Type<T>: Debug` is implied.
type Type<T: Clone>: Debug;
}If add_drop is true, we add a T: Drop bound for every type generic and associated type.
impl<'tcx, Id> ItemPredicates<'tcx, Id>where
Id: Clone,
impl<'tcx, Id> ItemPredicates<'tcx, Id>where
Id: Clone,
pub fn len(&self) -> usize
pub fn iter(&self) -> impl Iterator<Item = ItemPredicate<'tcx, Id>>
pub fn iter_trait_clauses( &self, ) -> impl Iterator<Item = (ItemPredicateId<Id>, Binder<TyCtxt<'tcx>, TraitRef<TyCtxt<'tcx>>>)>
pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut ItemPredicate<'tcx, Id>>
pub fn instantiate(
self,
tcx: TyCtxt<'tcx>,
args: &'tcx RawList<(), GenericArg<'tcx>>,
) -> ItemPredicates<'tcx, Id>
pub fn instantiate( self, tcx: TyCtxt<'tcx>, args: &'tcx RawList<(), GenericArg<'tcx>>, ) -> ItemPredicates<'tcx, Id>
Substitute all the predicates with these args.