Struct ItemPredicates
pub struct ItemPredicates<'tcx> {
required: bool,
next_id: usize,
pub predicates: Vec<ItemPredicate<'tcx>>,
}Fields
required: boolnext_id: usizepredicates: Vec<ItemPredicate<'tcx>>Implementations
impl<'tcx> ItemPredicates<'tcx>
impl<'tcx> ItemPredicates<'tcx>
pub fn new_unmapped( span: Span, predicates: impl IntoIterator<Item = Clause<'tcx>>, ) -> ItemPredicates<'tcx>
pub fn defined_on(
tcx: TyCtxt<'_>,
def_id: DefId,
required: bool,
) -> ItemPredicates<'_>
pub fn defined_on( tcx: TyCtxt<'_>, def_id: DefId, required: bool, ) -> ItemPredicates<'_>
Returns a list of type predicates for the definition with id def_id, including inferred
lifetime constraints. This is the basic list of predicates we use for essentially all
items.
pub fn required(
tcx: TyCtxt<'tcx>,
def_id: DefId,
options: &BoundsOptions,
) -> ItemPredicates<'tcx>
pub fn required( tcx: TyCtxt<'tcx>, def_id: DefId, options: &BoundsOptions, ) -> ItemPredicates<'tcx>
The predicates that must hold to mention this item (ignoring its parents). E.g.
ⓘ
// `U: OtherTrait` is required, `Self: Sized` is implied.
trait Trait<U: OtherTrait>: Sized {
// `T: Clone` is required, `Self::Type<T>: Debug` is implied.
type Type<T: Clone>: Debug;
}If add_drop is true, we add a T: Drop bound for every type generic.
pub fn required_recursively(
tcx: TyCtxt<'tcx>,
def_id: DefId,
options: &BoundsOptions,
) -> ItemPredicates<'tcx>
pub fn required_recursively( tcx: TyCtxt<'tcx>, def_id: DefId, options: &BoundsOptions, ) -> ItemPredicates<'tcx>
The predicates that must hold to mention this item, including its parents. E.g.
ⓘ
// `U: OtherTrait` is required.
trait Trait<U: OtherTrait> {
// `U: OtherTrait` and `T: Clone` are required.
type Type<T: Clone>;
}pub fn implied(
tcx: TyCtxt<'tcx>,
def_id: DefId,
options: &BoundsOptions,
) -> ItemPredicates<'tcx>
pub fn implied( tcx: TyCtxt<'tcx>, def_id: DefId, options: &BoundsOptions, ) -> ItemPredicates<'tcx>
The predicates that can be deduced from the presence of this item in a signature. We only
consider predicates implied by traits here, not implied bounds such as &'a T implying T: 'a. E.g.
ⓘ
// `U: OtherTrait` is required, `Self: Sized` is implied.
trait Trait<U: OtherTrait>: Sized {
// `T: Clone` is required, `Self::Type<T>: Debug` is implied.
type Type<T: Clone>: Debug;
}If add_drop is true, we add a T: Drop bound for every type generic and associated type.
pub fn iter(&self) -> impl Iterator<Item = ItemPredicate<'tcx>>
pub fn iter_trait_clauses( &self, ) -> impl Iterator<Item = (ItemPredicateId, Binder<TyCtxt<'tcx>, TraitRef<TyCtxt<'tcx>>>)>
pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut ItemPredicate<'tcx>>